// -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
/*
  Copyright (C) 2012-2019  Dieter Schärf <ds@dschaerf.de>

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

const Clutter = imports.gi.Clutter;
const Lang = imports.lang;
const Mainloop = imports.mainloop;
const Shell = imports.gi.Shell;
const St = imports.gi.St;

const ExtensionUtils = imports.misc.extensionUtils;
const Main = imports.ui.main;
const PanelMenu = imports.ui.panelMenu;
const PopupMenu = imports.ui.popupMenu;
const Util = imports.misc.util;

const Me = ExtensionUtils.getCurrentExtension();

const Convenience = Me.imports.convenience;
const Sensors = Me.imports.sensors;

const _ = Convenience.getTranslations();

const KEY_REFRESH_INTERVAL = 'refresh-interval';
const KEY_SHOW_FAHRENHEIT = 'show-fahrenheit';
const KEY_SHOW_TEMPERATURE = 'show-temperature';

function get_temperature_as_string(temperature) {
    if (!temperature) return 'N/A';
    if (Convenience.Settings.get_boolean(KEY_SHOW_FAHRENHEIT)) {
        return Math.round(temperature * 1.8 + 32).toString()+' \u00b0F';
    } else {
        return (Math.round(temperature * 10) / 10).toFixed(1).toString()+' \u00b0C';
    }
}

const TemperatureMenuItem = new Lang.Class({
    Name: 'TemperatureMenuItem',
    Extends: PopupMenu.PopupBaseMenuItem,

    _init: function(text, wantIcon, params) {
        this.parent(params);
        if (wantIcon) {
            this.icon = new St.Icon({ style_class: 'popup-menu-icon' });
            this.actor.add_child(this.icon);
        }
        this.label = new St.Label({ text: text, y_align: Clutter.ActorAlign.CENTER });
        this.actor.add_child(this.label);
        this.actor.add(new St.Bin(), { expand: true });
        this.status = new St.Label({ y_align: Clutter.ActorAlign.CENTER });
        this.actor.add(this.status);
        this.actor.label_actor = this.label;
        this.actor.connect('destroy', Lang.bind(this, function() {
            Convenience.Settings.disconnect(this._settingsChangedId);
        }));
    },

    setSelected: function(selected) {
        this.selected = selected;
        this.setOrnament(selected ? PopupMenu.Ornament.CHECK : PopupMenu.Ornament.NONE);
    },

    setStatus: function(text) {
        this.status.set_text(text);
    }
});

const TemperatureSubMenu = new Lang.Class({
    Name: 'TemperatureSubMenu',
    Extends: PopupMenu.PopupSubMenuMenuItem,

    _init: function(text, iconName, params) {
        this.parent(text, true);
        this.icon.icon_name = iconName;
        this._status = new St.Label({ y_expand: true, y_align: Clutter.ActorAlign.CENTER });
        let children = this.actor.get_n_children();
        this.actor.insert_child_at_index(this._status, children - 1);
        this._settingsChangedId = Convenience.Settings.connect('changed::'+KEY_SHOW_FAHRENHEIT, Lang.bind(this, this._onShowFahrenheit));
        this.actor.connect('destroy', Lang.bind(this, function() {
            Convenience.Settings.disconnect(this._settingsChangedId);
        }));
    },

    _onShowFahrenheit: function() {
        if (!this.actor.visible) return;
        this._setStatus(get_temperature_as_string(this._tempAverage));
        switch (this._tempStatus.Type) {
            case 'HWM':
                this._itemCurrent.setStatus(get_temperature_as_string(this._tempStatus.Current));
                if (this._itemMaximum) this._itemMaximum.setStatus(get_temperature_as_string(this._tempStatus.Maximum));
                if (this._itemCritical) this._itemCritical.setStatus(get_temperature_as_string(this._tempStatus.Critical));
                break;
            default:
                let menuItems = this.menu._getMenuItems();
                for (let i = 0; i < menuItems.length; i++) {
                    menuItems[i].setStatus(get_temperature_as_string(this._tempStatus[i].Current));
                }
        }
    },

    _setStatus: function(text) {
        this._status.set_text(text);
    },

    setTemperature: function(tempStatus) {
        this.menu.removeAll();
        this._tempAverage = null;
        if (tempStatus.length > 0 || tempStatus.Type !== undefined) switch (tempStatus.Type) {
            case 'HWM':
                this._itemCurrent = new TemperatureMenuItem(_("Current temperature"), false, { reactive: false });
                this.menu.addMenuItem(this._itemCurrent);
                if (tempStatus.Maximum) {
                    this._itemMaximum = new TemperatureMenuItem(_("Maximum temperature"), false, { reactive: false });
                    this.menu.addMenuItem(this._itemMaximum);
                }
                if (tempStatus.Critical) {
                    this._itemCritical = new TemperatureMenuItem(_("Critical temperature"), false, { reactive: false });
                    this.menu.addMenuItem(this._itemCritical);
                }
                this._tempAverage = tempStatus.Current;
                break;
            default:
                let average = 0;
                for (let i = 0; i < tempStatus.length; i++) {
                    average += tempStatus[i].Current;
                    this.menu.addMenuItem(new TemperatureMenuItem(tempStatus[i].Label, false, { reactive: false }));
                }
                this._tempAverage = average > 0 ? average / tempStatus.length : null;
        }
        this._tempStatus = tempStatus;
        this.actor.visible = this.menu.numMenuItems > 0;
        this._onShowFahrenheit();
    }
});

const TemperatureMenuButton = new Lang.Class({
    Name: 'TemperatureMenuButton',
    Extends: PanelMenu.Button,

    _init: function() {
        this.parent(0.5, _("Temperature"));
        this.menu.actor.add_style_class_name('dschaerf-menu');
        this._hbox = new St.BoxLayout({ style_class: 'panel-status-indicators-box' });
        this._icon = new St.Icon({ style_class: 'system-status-icon'});
        this._hbox.add_child(this._icon);
        this._label = new St.Label({ y_align: Clutter.ActorAlign.CENTER }),
        this._hbox.add_child(this._label);
        this._hbox.add_child(PopupMenu.arrowIcon(St.Side.BOTTOM));
        this.actor.add_child(this._hbox);
        this._itemSelected = null;
        this._itemSensors = new PopupMenu.PopupMenuSection();
        this.menu.addMenuItem(this._itemSensors);
        this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
        this._itemAverage = this._createMenuItem('Average', _("Average temperature"), 'utilities-system-monitor');
        this.menu.addMenuItem(this._itemAverage);
        this._itemHighest = this._createMenuItem('Highest', _("Highest temperature"), 'utilities-system-monitor');
        this.menu.addMenuItem(this._itemHighest);
        this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
        this._itemActions = new PopupMenu.PopupBaseMenuItem({ reactive: false, can_focus: false });
        this._actionTemperatureSettings = this._createActionButton('preferences-other-symbolic', _("Settings"));
        this._actionTemperatureSettings.connect('clicked', Lang.bind(this, this._onClickedTemperatureSettings));
        this._itemActions.actor.add(this._actionTemperatureSettings, { expand: true, x_fill: false });
        this._actionRefresh = this._createActionButton('view-refresh-symbolic', _("Settings"));
        this._actionRefresh.connect('clicked', Lang.bind(this, this._onClickedRefresh));
        this._refreshTimeoutId = null;
        this._itemActions.actor.add(this._actionRefresh, { expand: true, x_fill: false });
        this.menu.addMenuItem(this._itemActions);
        this._sensors = Sensors.getSensors();
        this._settingsChangedId = [
            Convenience.Settings.connect('changed::'+KEY_REFRESH_INTERVAL, Lang.bind(this, this._onRefreshInterval)),
            Convenience.Settings.connect('changed::'+KEY_SHOW_FAHRENHEIT, Lang.bind(this, this._onShowFahrenheit)),
            Convenience.Settings.connect('changed::'+KEY_SHOW_TEMPERATURE, Lang.bind(this, this._onShowTemperature))
        ];
        this._onRefreshInterval();
        this.actor.connect('destroy', Lang.bind(this, function() {
            if (this._refreshTimeoutId) Mainloop.source_remove(this._refreshTimeoutId);
            this._settingsChangedId.forEach(function(connection) {
                Convenience.Settings.disconnect(connection);
            });
        }));
    },

    _createActionButton: function(iconName, accessibleName) {
        let icon = new St.Button({ reactive: true, can_focus: true, track_hover: true, accessible_name: accessibleName, style_class: 'system-menu-action' });
        icon.child = new St.Icon({ icon_name: iconName });
        return icon;
    },

    _createMenuItem: function(id, text, iconName) {
        let item = new TemperatureMenuItem(text, true);
        item.icon.icon_name = iconName;
        item.connect('activate', Lang.bind(this, function() {
            if (this._itemSelected && this._itemSelected !== item) this._itemSelected.setSelected(false);
            item.setSelected(!item.selected);
            this._itemSelected = item.selected ? item : null;
            Convenience.Settings.set_strv(KEY_SHOW_TEMPERATURE, [ item.selected ? id : '' ]);
        }));
        let showTemperature = Convenience.Settings.get_strv(KEY_SHOW_TEMPERATURE);
        item.setSelected(showTemperature.indexOf(id) >= 0);
        if (item.selected) this._itemSelected = item;
        return item;
    },

    _onClickedRefresh: function() {
        this._onRefreshInterval();
    },

    _onClickedTemperatureSettings: function() {
        this.menu.itemActivated();
        Util.spawnCommandLine('gnome-shell-extension-prefs '+Me.metadata['uuid']);
        Main.overview.hide();
    },

    _onRefreshInterval: function() {
        if (this._refreshTimeoutId) Mainloop.source_remove(this._refreshTimeoutId);
        this._updateSensors();
        this._refreshTimeoutId = Mainloop.timeout_add_seconds(Convenience.Settings.get_string(KEY_REFRESH_INTERVAL), Lang.bind(this, this._updateSensors));
    },

    _onShowFahrenheit: function() {
        this._itemAverage.setStatus(get_temperature_as_string(this._tempAverage));
        this._itemHighest.setStatus(get_temperature_as_string(this._tempHighest));
        this._onShowTemperature();
    },

    _onShowTemperature: function() {
        if (this._itemSelected) this._label.set_text(this._itemSelected.status.get_text());
        this._label.visible = this._itemSelected !== null;
    },

    _updateSensors: function() {
        let iconName = 'utilities-system-monitor-symbolic';
        let item;
        let numSensorInfo = 0;
        let sensorStatus = {
            Counter: 0,
            Current: null,
            Highest: null
        };
        this._itemSensors.removeAll();
        this._sensors.forEach(function(sensor) {
            let sensorInfo = [];
            if (sensor.available) {
                sensorInfo = sensor.sensorInfo;
                sensorInfo.forEach(function(info) {
                    numSensorInfo ++;
                    sensorStatus.Counter ++;
                    sensorStatus.Current += info.Current;
                    if (!sensorStatus.Highest || sensorStatus.Highest < info.Current) sensorStatus.Highest = info.Current;
                    if (info.Warning) iconName = 'dialog-warning';
                });
            }
            switch (sensor.Type) {
                case 'HWM':
                    sensorInfo.forEach(function(info) {
                        item = new TemperatureSubMenu(info.Label, info.Warning ? 'dialog-warning' : 'utilities-system-monitor');
                        item.setTemperature(info);
                        this._itemSensors.addMenuItem(item);
                    }, this);
                    if (numSensorInfo > 1) this._itemSensors.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
                    break;
                case 'HDD':
                    item = new TemperatureSubMenu(_("Hard disk drives"), 'drive-harddisk');
                    item.setTemperature(sensorInfo);
                    this._itemSensors.addMenuItem(item);
                    break;
                case 'VGA':
                    item = new TemperatureSubMenu(_("Graphics Cards"), 'video-display');
                    item.setTemperature(sensorInfo);
                    this._itemSensors.addMenuItem(item);
                    break;
                default: // Do nothing
            }
        }, this);
        if (sensorStatus.Counter < 1) {
            iconName = 'dialog-warning-symbolic';
            this._itemSensors.addMenuItem(new PopupMenu.PopupMenuItem(_("No supported temperature sensors available"), { reactive: false, can_focus: false }));
        }
        this._tempAverage = sensorStatus.Counter > 0 ? sensorStatus.Current / sensorStatus.Counter : null;
        this._tempHighest = sensorStatus.Highest;
        this._icon.icon_name = iconName;
        this._onShowFahrenheit();
    }
});

function init(metadata) {
    Convenience.initTranslations();
}

let _temperature;

function enable() {
    _temperature = Main.panel.addToStatusArea('dschaerfTemperature', new TemperatureMenuButton(), 1, 'center');
    //  NOTE: Activate the next line for testing purposes only
    Main.panelTemperature = _temperature;
}

function disable() {
    _temperature.destroy();
}
