// -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
/*
  Copyright (C) 2012-2019  Dieter Schärf <ds@dschaerf.de>

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

const GLib = imports.gi.GLib;
const Lang = imports.lang;

const ExtensionUtils = imports.misc.extensionUtils;

const Me = ExtensionUtils.getCurrentExtension();

const Convenience = Me.imports.convenience;

const _ = Convenience.getTranslations();

function get_command_line_string(command, index, seperator) {
    let output = GLib.spawn_command_line_sync(command);
    if (index) output = output[index];
    output = Convenience.checkVersion([ '3.30' ]) ? String.fromCharCode.apply(null, output) : output.toString();
    if (seperator) output = output.split(seperator);
    return output;
}

const SensorHDD = new Lang.Class({
    Name: 'Temperature.SensorHDD',

    _init: function() {
        this.Type = 'HDD';
    },

    _getSensorOutput: function(sensor) {
        let content = [];
        switch (sensor) {
            case 'HDDTempCommand':
            case 'HDDTempDaemon':
                let pos = sensor === 'HDDTempDaemon' ? 1 : 0;
                let sep = sensor === 'HDDTempDaemon' ? '|' : ': ';
                let lines = get_command_line_string(this._sensorCommands[sensor], 1, '\n');
                lines.forEach(function(line) {
                    let info = line.split(sep);
                    let current = parseFloat(info[pos + 2]);
                    if (info[4] && info[4] === 'F') current = (current - 32) * 5 / 9;
                    if (current > 0) content.push({
                        Device : info[pos],
                        Model  : info[pos + 1],
                        Current: current
                    });
                });
                break;
            case 'UDisksCTL':
                let udisksStatus = get_command_line_string(this._sensorCommands[sensor]+' status' , 1, '\n');
                udisksStatus.forEach(function(status) {
                    let device = '/dev/'+status.substr(status.length - 8, 8).trim();
                    if (device.length === 8) {
                        let udisksInfoBlockDevice = get_command_line_string(this._sensorCommands[sensor]+' info --block-device='+device, 1, '\n');
                        udisksInfoBlockDevice.forEach(function(blockDevice) {
                            if (blockDevice.indexOf('Drive:') >= 0) {
                                let drive = blockDevice.split('Drive:')[1].split(/'/)[1].split('/')[5];
                                let model;
                                let current = 0;
                                let udisksInfoDrive = get_command_line_string(this._sensorCommands[sensor]+' info --drive="'+drive+'"', 1, '\n');
                                udisksInfoDrive.forEach(function(info) {
                                    if (info.indexOf('Model:') >= 0) model = info.split(':')[1].trim();
                                    if (info.indexOf('SmartTemperature:') >= 0) current = parseFloat(info.split(':')[1]);
                                });
                                if (current > 0) content.push({
                                    Device : device,
                                    Model  : model,
                                    Current: current - 273.15
                                });
                            }
                        }, this);
                    }
                }, this);
                break;
            default: // Do nothing
        }
        return content;
    },

    get available() {
        this._sensorCommands = {};
        let commandHDDTemp = GLib.find_program_in_path('hddtemp');
        if (commandHDDTemp) this._sensorCommands.HDDTempCommand = commandHDDTemp+' --unit=C /dev/sda /dev/sdb /dev/sdc /dev/sdd';
        let commandNC = GLib.find_program_in_path('nc');
        let commandPS = GLib.find_program_in_path('ps');
        let pid = get_command_line_string('pidof hddtemp', 1, null);
        if (pid.length > 0 && commandNC && commandPS) {
            let output = get_command_line_string('ps --pid='+pid+' -o args=', null, null);
            let port = output.split(' -p ')[1].split(' ')[0];
            this._sensorCommands.HDDTempDaemon = commandNC+' localhost '+port;
        }
        let commandUDisksCTL = GLib.find_program_in_path('udisksctl');
        if (commandUDisksCTL) this._sensorCommands.UDisksCTL = commandUDisksCTL;
        return Object.keys(this._sensorCommands).length > 0;
    },

    get sensorInfo() {
        let sensorInfo = [];
        let sensorOutput;
        for (let sensor in this._sensorCommands) {
            sensorOutput = this._getSensorOutput(sensor);
            sensorOutput.forEach(function(output) {
                let duplicate = false;
                sensorInfo.forEach(function(info) {
                    if (info.Label.indexOf(output.Model) >= 0) duplicate = true;
                });
                if (!duplicate) sensorInfo.push({
                    Type    : 'HDD',
                    Label   : output.Model+'  [ %s ]'.format(output.Device),
                    Current : output.Current,
                    Maximum : null,
                    Critical: null,
                    Warning : false
                });
            });
        }
        sensorInfo.sort(function(a, b) {
            return GLib.utf8_collate(a.Label, b.Label);
        });
        return sensorInfo;
    }
});

const SensorHWM = new Lang.Class({
    Name: 'Temperature.SensorHWM',

    _init: function() {
        this.Type = 'HWM';
    },

    _getContent: function(fileName, asString) {
        if (!GLib.file_test(fileName, GLib.FileTest.EXISTS)) return null;
        try { 
            let content = GLib.file_get_contents(fileName);
            if (asString) {
                return _(content[1].toString().split('\n')[0]);
            } else {
                return parseInt(content[1]) / 1000;
            }
        } catch(error) {
            return null;
        }
    },

    get available() {
        this._sensorPaths = [];
        for (let i = 0; ; i++) {
            let hwmonPath = '/sys/class/hwmon/hwmon'+i+'/';
            let hwmonPathDevice = hwmonPath+'device/';
            if (GLib.file_test(hwmonPath+'name', GLib.FileTest.EXISTS)) {
                this._sensorPaths.push(hwmonPath);
            } else if (GLib.file_test(hwmonPathDevice+'name', GLib.FileTest.EXISTS)) {
                this._sensorPaths.push(hwmonPathDevice);
            } else {
                if (i > 1) break;
            }
        }
        return this._sensorPaths.length > 0;
    },

    get sensorInfo() {
        let sensorInfo = [];
        this._sensorPaths.forEach(function(path) {
            let hwmonName = this._getContent(path+'name', true);
            if (hwmonName) for (let i = 1; ; i++) {
                let hwmonTemp = path+'temp'+i;
                let hwmonInput = this._getContent(hwmonTemp+'_input', false);
                if (!hwmonInput && i > 2) break;
                if (!hwmonInput) continue;
                let hwmonLabel = this._getContent(hwmonTemp+'_label', true);
                let hwmonMax = this._getContent(hwmonTemp+'_max', false);
                let hwmonCrit = this._getContent(hwmonTemp+'_crit', false);
                //TODO: Add unknow sensors
                switch (hwmonName) {
                    case 'acpitz':
                        hwmonLabel = _("ACPI thermal device  [ %s ]").format(i);
                        break;
                    case 'coretemp': // All Intel Core family
                        hwmonLabel = _("Processor - Intel  [ %s ]").format(hwmonLabel);
                        break;
                    case 'k8temp':   // AMD Athlon64/FX or Opteron CPUs
                        let core = i < 2 ? 1 : 0;
                        let place = i == 2 || i === 4 ? 1 : 0;
                        hwmonLabel = _("Processor - AMD K8  [ Core %s/%s ]").format(core, place);
                        break;
                    case 'k10temp':  // AMD Family 10h/11h/12h/14h/15h/16h/17h/18h/19h
                        let k10Input = this._getContent(path+'temp2_input', false);
                        if (k10Input) {
                            hwmonInput = k10Input;
                            let k10Label = this._getContent(path+'temp2_label', true);
                            if (k10Label) hwmonLabel = k10Label;
                            i++;
                        }
                        hwmonLabel = _("Processor - AMD K10  [ %s ]").format(hwmonLabel);
                        break;
                    case 'amdgpu':   // AMD Radeon GPUs
                    case 'nouveau':  // NV43+
                        hwmonLabel = _("Graphic card  [ %s ]").format(hwmonName);
                        break;
                    default:         // Unknown
                        hwmonLabel = hwmonName+'  [ %s ]'.format(hwmonLabel || i);
                }
                hwmonInput = hwmonInput;
                let hwmonWarning = false;
                if (hwmonMax && hwmonInput > hwmonMax * 0.9 || hwmonCrit && hwmonInput > hwmonCrit * 0.8) hwmonWarning = true;
                sensorInfo.push({
                    Type    : 'HWM',
                    Label   : hwmonLabel,
                    Current : hwmonInput,
                    Maximum : hwmonMax,
                    Critical: hwmonCrit,
                    Warning : hwmonWarning
                });
            }
        }, this);
        return sensorInfo;
    }
});

const SensorVGA = new Lang.Class({
    Name: 'Temperature.SensorVGA',

    _init: function() {
        this.Type = 'VGA';
    },

    _getSensorOutput: function(sensor) {
        let sensorInfo = [];
        switch (sensor) {
            case 'ATIConfig':
            case 'NVidiaSMI':
                let adapter = temperature = null;
                let lines = get_command_line_string(this._sensorCommands[sensor], 1, '\n');
                lines.forEach(function(line) {
                    if (line.indexOf('Default Adapter') >= 0) adapter = line.split(' - ')[1];
                    if (line.indexOf('Product Name') >= 0) adapter = line.split(' : ')[1];
                    if (line.indexOf('Temperature') >= 0) temperature = line.split(/ - | : /)[1];
                    if (adapter !== null && temperature !== null) {
                        sensorInfo.push({
                            Adapter: adapter,
                            Current: parseFloat(temperature)
                        });
                        adapter = temperature = null;
                    }
                });
                break;
            default: // Do nothing
        }
        return sensorInfo;
    },

    get available() {
        this._sensorCommands = {};
        let commandATIConfig = GLib.find_program_in_path('aticonfig');
        if (commandATIConfig) this._sensorCommands.ATIConfig = commandATIConfig+' --odgt';
        let commandNVidiaSMI = GLib.find_program_in_path('nvidia-smi');
        if (commandNVidiaSMI) this._sensorCommands.NVidiaSMI = commandNVidiaSMI+' -q -a';
        return Object.keys(this._sensorCommands).length > 0;
    },

    get sensorInfo() {
        let sensorInfo = [];
        let sensorOutput;
        for (let sensor in this._sensorCommands) {
            sensorOutput = this._getSensorOutput(sensor);
            sensorOutput.forEach(function(output) {
                let duplicate = false;
                sensorInfo.forEach(function(info) {
                    if (info.Label.indexOf(output.Adapter) >= 0) duplicate = true;
                });
                if (!duplicate) sensorInfo.push({
                    Type    : 'VGA',
                    Label   : output.Adapter,
                    Current : output.Current,
                    Maximum : null,
                    Critical: null,
                    Warning : false
                });
            });
        }
        sensorInfo.sort(function(a, b) {
            return GLib.utf8_collate(a.Label, b.Label);
        });
        return sensorInfo;
    }
});

let _sensors = [];

function getSensors() {
    if (_sensors.length === 0) {
        _sensors.push(new SensorHWM);
        _sensors.push(new SensorVGA);
        _sensors.push(new SensorHDD);
    }
    return _sensors;
}
